def string_to_netstring_ascii(string: str, *args):
    """Convert the given string to a netstring (and return it's ascii representation)."""
    netstring = f'{len(string)}:{string},'

    if any(args):
        for arg in args:
            netstring += string_to_netstring_ascii(arg)
        netstring = string_to_netstring_ascii(netstring)

    return netstring


def string_to_netstring_hex(string: str, *args):
    """Convert the given string to a netstring (and return it's hex representation)."""
    netstring_ascii = string_to_netstring_ascii(string, *args)
    nestring_hex = netstring_ascii_to_netstring_hex(netstring_ascii)
    return nestring_hex


def netstring_ascii_to_netstring_hex(netstring_ascii: str):
    """Convert a netstring (represented as ascii) to its hex representation."""
    from democritus_strings import string_to_hex

    nestring_hex = string_to_hex(netstring_ascii, seperator=' ')
    return nestring_hex


def netstring_hex_to_netstring_ascii(netstring_hex: str):
    """Convert a netstring (represented as hex) to its ascii representation."""
    from democritus_strings import hex_to_string

    nestring_ascii = hex_to_string(netstring_hex)
    return nestring_ascii


def netstring_ascii_to_string(netstring_ascii: str):
    """Get the string portion of the given netstring (represented as ascii)."""
    # TODO: there should be a construct for splitting and getting the first item of the split
    length = netstring_ascii.split(':')[0]
    ascii_string = ':'.join(netstring_ascii.split(':')[1:])
    # this will remove the trailing comma
    ascii_string = ascii_string[:-1]
    return ascii_string


def netstring_hex_to_string(netstring_hex: str):
    """Get the string portion of the given netstring (represented as hex)."""
    netstring_ascii = netstring_hex_to_netstring_ascii(netstring_hex)
    ascii_string = netstring_ascii_to_string(netstring_ascii)
    return ascii_string



try:
    from importlib.metadata import version, PackageNotFoundError
except ImportError:
    from importlib_metadata import version, PackageNotFoundError

try:
    __version__ = version('democritus_netstrings')
except PackageNotFoundError:
    message = 'Unable to find a version number for "democritus_netstrings". This likely means the library was not installed properly. Please re-install it and, if the problem persists, raise an issue here: https://github.com/democritus-project/democritus-netstrings/issues.'
    print(message)

__author__ = '''Floyd Hightower'''
__email__ = 'floyd.hightower27@gmail.com'

from .netstrings import *



from democritus_netstrings import (
    string_to_netstring_ascii,
    string_to_netstring_hex,
    netstring_ascii_to_string,
    netstring_hex_to_string,
    netstring_ascii_to_netstring_hex,
    netstring_hex_to_netstring_ascii,
)


def test_netstring_ascii_to_netstring_hex_1():
    results = netstring_ascii_to_netstring_hex('12:hello world!,')
    assert results == '31 32 3a 68 65 6c 6c 6f 20 77 6f 72 6c 64 21 2c'


def test_netstring_hex_to_netstring_ascii_1():
    results = netstring_hex_to_netstring_ascii('31 32 3a 68 65 6c 6c 6f 20 77 6f 72 6c 64 21 2c')
    assert results == '12:hello world!,'


def test_string_to_netstring_ascii_1():
    results = string_to_netstring_ascii('hello world!')
    assert results == '12:hello world!,'

    results = string_to_netstring_ascii('')
    assert results == '0:,'

    results = string_to_netstring_ascii('hello', 'world!')
    assert results == '17:5:hello,6:world!,,'

    results = string_to_netstring_ascii('hello', 'beautiful', 'world!')
    assert results == '29:5:hello,9:beautiful,6:world!,,'

    results = string_to_netstring_ascii('hello, world!')
    assert results == '13:hello, world!,'


def test_string_to_netstring_hex_1():
    results = string_to_netstring_hex('hello world!')
    assert results == '31 32 3a 68 65 6c 6c 6f 20 77 6f 72 6c 64 21 2c'

    results = string_to_netstring_hex('')
    assert results == '30 3a 2c'

    results = string_to_netstring_hex('hello', 'beautiful', 'world!')
    assert (
        results == '32 39 3a 35 3a 68 65 6c 6c 6f 2c 39 3a 62 65 61 75 74 69 66 75 6c 2c 36 3a 77 6f 72 6c 64 21 2c 2c'
    )


def test_netstring_ascii_to_string_1():
    results = netstring_ascii_to_string('12:hello world!,')
    assert results == 'hello world!'

    results = netstring_ascii_to_string('13:hello, world!,')
    assert results == 'hello, world!'


def test_netstring_hex_to_string_1():
    results = netstring_hex_to_string('31 32 3a 68 65 6c 6c 6f 20 77 6f 72 6c 64 21 2c')
    assert results == 'hello world!'

    results = netstring_hex_to_string(
        '32 39 3a 35 3a 68 65 6c 6c 6f 2c 39 3a 62 65 61 75 74 69 66 75 6c 2c 36 3a 77 6f 72 6c 64 21 2c 2c'
    )
    assert results == '5:hello,9:beautiful,6:world!,'
